/*******************************************************
 *
 * $Id: Customization.h,v 1.11 2003/03/17 22:37:35 teacy Exp $
 *
 * This file has been placed in the public domain.
 *
 *******************************************************/



#ifndef CUSTOMIZATION_H
#define CUSTOMIZATION_H

#include <PalmOS.h>


/********************************************************
 * Modify the preprocessor settings in the first section
 * of this file to adjust some extremely low-level 
 * properties of the
 * Razor! engine (use only OS API, run in profiling mode, 
 * etc.)
 * Use the static constants in order
 * to specify the values of all the other available
 * parameters of the engine.
 ********************************************************/



/*
 * Preprocessor settings
 */

//============================================================================================================
//======================== Special 'modes' of the engine =====================================================
//============================================================================================================

// Shall the application be compiled into a 'pure' version?
// Purity means: 1) use only the official Palm OS API, 2) do not use knowledge about
// the device's processor.
//#define MODE_PURITY

// Shall the application be run in a special profiling mode? (see below)
//#define MODE_PROFILING

// Shall the application be compiled for a specific CPU?
#define MODE_M68K           // Use this to produce code for devices with a real or emulated m68k CPU.
//#define  MODE_ARM            // Use this to produce native ARM code for future devices (not supported yet).

//============================================================================================================
//======================== Application specific behavior variables  ==========================================
//============================================================================================================

/*
 * Application settings
 */

/// The framework requires a minimum of v3.0, but you can raise the limit as required by your own application.
static const UInt32 appMinRomVersion = sysMakeROMVersion(3,0,0,sysROMStageDevelopment,0);
#ifdef USE_SDK50
static const UInt32 appMaxRomVersion = sysMakeROMVersion(100,0,0,sysROMStageDevelopment,0);
#else
// Do not touch this, since SDK 4.0 builds will not work well on OS5 devices.
static const UInt32 appMaxRomVersion = sysMakeROMVersion(4,9,0,sysROMStageDevelopment,0);
#endif


/*
 * Profiling settings
 */

// If MODE_PROFILING is defined, profiling starts at period 
// 'profilingBegin', and  ends at period 'profilingEnd', 
// dumping a file called <profilingFile>. 
// Enabling profiling also suppresses the save and restore 
// mechanism for application state, and it ignores all user input.
#ifdef MODE_PROFILING
static const UInt32 profilingBegin = 5;
static const UInt32 profilingEnd = 250;
static const Char   *profilingFile = "SimpleDemoProfile";

// If your application crashes while in profiling mode,
// but does not crash while in normal mode, you should
// attempt to raise these values (Just double them, and
// see if that solves the problem).
static const long   profilingMaxCalls = 0x40000;
static const long   profilingMaxDepth = 500;

#endif



#endif  // CUSTOMIZATION_H
