/*******************************************************
 *
 * $Id: RectangleScreen.cpp,v 1.7 2003/05/29 15:41:13 teacy Exp $
 *
 * This file has been placed in the public domain.
 *
 *******************************************************/


#include <PalmOS.h>

#include "RectangleScreen.h"
#include "sound/SoundEngine.h"



RectangleScreen::RectangleScreen(const StateDescriptor *screenState) : Screen()
{
	state = (RectangleScreenState *)(screenState->state);
	exitScreen = false;
}


void RectangleScreen::init(const Canvas *, const Canvas *)
{
}


RectangleScreen::~RectangleScreen()
{
}


UInt32 RectangleScreen::getAdvanceTimeInterval()
{
	return 1;
}

	
Boolean RectangleScreen::nextPeriod()
{
	state->periodCounter++;

	return exitScreen;
}


void RectangleScreen::getViewportProperties(const Canvas *displayCanvas, ViewportProperties& viewport) const
{
	viewport.region.topLeft.x =  0;
	viewport.region.topLeft.y = 24;
	viewport.region.extent.x = displayCanvas->width;
	viewport.region.extent.y = displayCanvas->height - 48;
}


void RectangleScreen::drawBackground(const Canvas *viewportCanvas) const
{
	viewportCanvas->fillRectangle(Color::WHITE, 0, 0, viewportCanvas->width - 1, viewportCanvas->height - 1);
}


void RectangleScreen::drawAction(const Canvas *viewportCanvas, RectangleType *bounds)
{
	bounds->topLeft.x = (state->periodCounter * 37) % (viewportCanvas->width - 40);
	bounds->topLeft.y = ((state->periodCounter * 51) % (viewportCanvas->height - 40));
	bounds->extent.x = state->periodCounter & 63;
	bounds->extent.y = state->periodCounter & 63;

	// Calculate RGB color values from current frame number
	const Color color = Color(state->periodCounter & 255, 255 - ((state->periodCounter >> 1) & 255), (state->periodCounter << 2) & 255);
	
	viewportCanvas->fillRectangle(color, bounds->topLeft.x, bounds->topLeft.y, (bounds->topLeft.x + bounds->extent.x) - 1, (bounds->topLeft.y + bounds->extent.y) - 1);
}


void RectangleScreen::drawFixedOverlays(const Canvas *displayCanvas) const
{
	displayCanvas->fillRectangle(Color::BLACK, 0, 0, displayCanvas->width - 1, 23);
	displayCanvas->fillRectangle(Color::BLACK, 0, displayCanvas->height - 24, displayCanvas->width - 1, displayCanvas->height - 1);
}


void RectangleScreen::drawDynamicOverlays(const Canvas *) const
{
	// This screen has no dynamic overlays
}


void RectangleScreen::receiveEvent(const EventType* evtPtr)
{
	if (evtPtr->eType == penUpEvent)
	{
		SoundEngine::playFx(0);
		exitScreen = true;
	}
}


StateDescriptor *RectangleScreen::createStateDescriptor()
{	
	createState(RectangleScreenState);

	// Initialize state
	state->periodCounter = 0;

	return stateDescriptor;
}
