/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Canvas.cpp,v 1.5 2003/05/30 12:30:20 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include "./HostControl.h"
#include <PalmOS.h>

#include "gfx/Display.h"
#include "gfx/Canvas.h"
#include "gfx/Blitter.h"
#include "gfx/Flatshader.h"

#include "Device.h"

using namespace prvrazor;


BitmapArray::BitmapArray(UInt16 numBitmaps)
{
	this->numBitmaps = numBitmaps;
}


BitmapArray::~BitmapArray()
{
}



SaveBufferArray::SaveBufferArray(UInt16 numBuffers, Coord width, Coord height)
{
    this->numBuffers = numBuffers;
    this->width = width;
    this->height = height;
}


SaveBufferArray::~SaveBufferArray()
{
}




Canvas::Canvas(Coord width, Coord height, CoordSystem coordSystem)
{
    this->coordSystem = coordSystem;

	this->width = width;
	this->height = height;

    this->shiftFactor = Canvas::calcShiftFactor(coordSystem);
    
    rows = height << shiftFactor;
    cols = width << shiftFactor;
}


Canvas::~Canvas() 
{
}


void Canvas::setCoordSystem(CoordSystem coordSystem)
{
    this->coordSystem = coordSystem;

    this->shiftFactor = Canvas::calcShiftFactor(coordSystem);

    height = rows >> shiftFactor;
    width = cols >> shiftFactor;
}


const BitmapArray *Canvas::prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const
{
	return blitter->prepareTransparentBitmaps(numBitmaps, bitmapID, maskID);
}

const BitmapArray *Canvas::prepareTransparentBitmap(DmResID bitmapID, DmResID maskID) const
{
	return blitter->prepareTransparentBitmaps(1, bitmapID, maskID);
}

const BitmapArray *Canvas::prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const
{
	return blitter->prepareSolidBitmaps(numBitmaps, bitmapID);
}

const BitmapArray *Canvas::prepareSolidBitmap(DmResID bitmapID) const
{
	return blitter->prepareSolidBitmaps(1, bitmapID);
}

void Canvas::drawBitmap(UInt16 index, const BitmapArray *bitmapArray, Coord x, Coord y) const
{
	bitmapArray->draw(this, index, x, y);
}

void Canvas::drawBitmap(const BitmapArray *bitmapArray, Coord x, Coord y) const
{
	bitmapArray->draw(this, 0, x, y);
}

void Canvas::drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const
{
	blitter->drawText(textColor, bgColor, font, chars, len, x, y);
}


void Canvas::fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const
{
	blitter->fillRectangle(color, topX, topY, bottomX, bottomY);
}


void Canvas::drawPolygon(const xy *e, int edgeCount, const Color color) const
{
	Flatshader::drawPolygon(this, e, edgeCount, color);
}



SaveBufferArray *Canvas::prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const
{
	return blitter->prepareSaveBuffers(numBuffers, width, height);
}


void Canvas::saveBitmap(UInt16 index, SaveBufferArray *bufferArray, Coord x, Coord y) const
{
	bufferArray->save(this, index, x, y);
}


void Canvas::restoreBitmap(UInt16 index, SaveBufferArray *bufferArray) const
{
	bufferArray->restore(this, index);
}


int Canvas::calcShiftFactor(UInt16 coordSystem)
{
    int retValue = 0;

    switch(Display::density)
    {
        case Display::DENSITY_SINGLE:
            switch(coordSystem)
            {
                case Canvas::COORDS_NATIVE:
                    retValue = 0;
                break;
                case Canvas::COORDS_STANDARD:
                    retValue = 0;
                break;
            }
        break;
        case Display::DENSITY_DOUBLE:
            switch(coordSystem)
            {
                case Canvas::COORDS_NATIVE:
                    retValue = 0;
                break;
                case Canvas::COORDS_STANDARD:
                    retValue = 1;
                break;
            }
        break;        
    }

    return retValue;
}
