/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Armlet.cpp,v 1.2 2003/03/17 22:28:55 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#include "Customization.h"

#ifdef USE_SDK50

#include <PalmOS.h>
#include <PceNativeCall.h>
#include "Armlet.h"
#include "Device.h"

Armlet::Armlet(DmResID resID, const char* DLLEntryPointP)
{
	this->DLLEntryPointP = NULL;
	this->armH = 0;
	this->armP = NULL;
	this->available = false;

	// get the processor type
	UInt32    processorType;
	FtrGet(sysFileCSystem, sysFtrNumProcessorID, &processorType);
	
	if (processorType >= sysFtrNumProcessorARM720T && processorType <= sysFtrNumProcessorARM710A)
	{
		// running on ARM; call the actual ARM resource
		this->armH = DmGetResource('ARMC', resID);
		if (this->armH != NULL)
		{
			this->armP = MemHandleLock(this->armH);
			this->available = true;
		}
	}
	else if (processorType == sysFtrNumProcessorx86)
	{
		// running on Simulator; remember the DLL
		if (DLLEntryPointP != NULL)
		{
			this->available = true;
			this->DLLEntryPointP = DLLEntryPointP;
		}
	}
	else
	{
		// some other processor
	}	
}

Armlet::~Armlet()
{
	if (armH != 0)
	{
		MemHandleUnlock(armH);
		DmReleaseResource(armH);
	}
}

UInt32 Armlet::invoke(void *userData) const
{
	UInt32 result = 0L;

	if (armH != 0)
	{
		result = PceNativeCall((NativeFuncType*)armP, userData);
	}
	else
	{
		// running on Simulator; call the DLL
		result = PceNativeCall( (NativeFuncType*)DLLEntryPointP, userData);
	}

	return result;
}


Boolean Armlet::isAvailable() const
{
	return available;
}

#endif
