/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Flatshader.armlet.c,v 1.2 2003/03/17 22:33:39 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#pragma thumb off

#include <PceNativeCall.h>
#include "endianutils.h"
#include "ArmletTypes.h"
#include "gfx/FlatshaderArmlet.h"

typedef struct {Int16 x, y;} xy;

/**
 * Draw a list of lines to canvas bitmap
 */
#ifdef __GNUC__
UInt32 NativeFunction(const void *emulStateP, void *userData68KP, Call68KFuncType *call68KFuncP)
#elif __MWERKS__
unsigned long ARMlet_Main(const void *emulStateP, void *userData68KP, Call68KFuncType *call68KFuncP)
#else
#error You are using an unsupported compiler
#endif
{
	const FlatShaderParamType *params = (FlatShaderParamType *)userData68KP;
	
	Int16 *__restrict minx = (Int16 *)ReadUnaligned32(&(params->minxptr));
	Int16 *__restrict maxx = (Int16 *)ReadUnaligned32(&(params->maxxptr));	

	const UInt32 vertexCount = (UInt32)ReadUnaligned32(&(params->vertexCount));

	const UInt32 rowBytes = (UInt32)ReadUnaligned32(&(params->rowBytes));
	UInt8 *drawBits = (UInt8 *)ReadUnaligned32(&(params->drawBits));
	const Int32 clipxr = (UInt32)ReadUnaligned32(&(params->clipxr));
	const Int32 clipyo = (UInt32)ReadUnaligned32(&(params->clipyo));
	const UInt32 colorIdx = (UInt32)ReadUnaligned32(&(params->colorIdx));
	xy *e = (xy *)ReadUnaligned32(&(params->e));		
	const UInt32 shiftFactor = (UInt32)ReadUnaligned32(&(params->shiftFactor));

	// Fix endian-ness of vertex coordinates
	for(UInt16 vertexIdx = 0; vertexIdx < vertexCount; vertexIdx++)
	{
		e[vertexIdx].x = ByteSwap16(e[vertexIdx].x) << shiftFactor;
		e[vertexIdx].y = ByteSwap16(e[vertexIdx].y) << shiftFactor;
	}

	// Determine the miny/maxy
	int miny, maxy;
	miny = maxy = e[0].y;
	
	for(int vertexIdx = 1; vertexIdx < vertexCount; vertexIdx++)
	{
		int vy = e[vertexIdx].y;
		if (vy < miny)
		{
			miny = vy;
		}		
		else if (vy > maxy)
		{
			maxy = vy;
		}
	}

	if (miny < 0)
	{
		miny = 0;
	}
	else if (miny > clipyo)
	{
		return 0L;
	}
	
	if (maxy > clipyo)
	{
		maxy = clipyo;
	}
	else if (maxy < 0)
	{
		return 0L;
	}

	// Initialize the minx/maxx fields
	{
		Int16 *__restrict minxptr = &minx[miny];
		Int16 *__restrict maxxptr = &maxx[miny];
	
		for(int y = maxy - miny; y >= 0; y--)
		{
			*minxptr++ = 32000;
			*maxxptr++ = 0;
		}
	}

	// Iterate over the edges of the polygon
	int vertexIdx = vertexCount;
	int flago=0, flagu=0;
	
	do 
	{
		int vIdx = (vertexIdx == vertexCount) ? 0 : vertexIdx;

		int y1 = e[vIdx].y;
		int x1 = e[vIdx].x;

		int y2 = e[vertexIdx-1].y;
		int x2 = e[vertexIdx-1].x;

		// horizontal edge
		if (y1 == y2) 
		{
			if (y1 > clipyo) { flago++; continue; }
			if (y1 < 0) { flagu++; continue; }
			if (x1 > x2) { int swap = x1; x1 = x2; x2 = swap; }
			if (x1 < 0) x1 = 0;
			if (x1 < minx[y1]) minx[y1] = x1;

			if (x2 > clipxr) x2 = clipxr;
			if (x2 > maxx[y1]) maxx[y1] = x2;
		}
		// vertical edge
		else if (x1 == x2)
		{
			if (x1 < 0) 
			{
				x1 = 0;
			}
			else if (x1 > clipxr) 
			{
				x1 = clipxr;
			}

			if (y1 > y2) { int yswap = y1; y1 = y2; y2 = yswap; }
			if (y1 > clipyo) {flago++; continue;} // Trivial rejection
			if (y2 < 0) {flagu++; continue;} // Trivial rejection
			if (y1 < 0) y1 = 0;
			if (y2 > clipyo) y2 = clipyo;
			
			int rowcount = y2-y1;
			Int16 *__restrict minxptr = &minx[y1];
			Int16 *__restrict maxxptr = &maxx[y1];
			do 
			{
				if (x1 < *minxptr) *minxptr = x1;
				if (x1 > *maxxptr) *maxxptr = x1;
				minxptr++;
				maxxptr++;
			} while(--rowcount >= 0);
		}
		// diagonal edge
		else 
		{
			// Proprietary clipping algorithm
			if (y1 > y2)
				{ int swap = x1; x1 = x2; x2 = swap;
					swap = y1; y1 = y2; y2 = swap; }
			if (y1 > clipyo) {flago++; continue;} // Trivial rejection
			if (y2 < 0) {flagu++; continue;} // Trivial rejection

			if (y1 < 0) 
			{
				long m = ((x2-x1) << 7) / (y2-y1);
				x1 -= (m * y1) >> 7;
				y1 = 0;
			}
			if (y2 > clipyo) 
			{
				long m = (((long)(x2-x1)) << 7) / (y2-y1);
				x2 += (m * (clipyo-y2)) >> 7;
				y2 = clipyo;
			}

			if (y1 == y2) 
			{
				// Edge has degenerated into a horizontal edge b/c of clipping
				if (x1 < 0) 
				{
					x1 = 0;
				}
				else if (x1 > clipxr) 
				{
					x1 = clipxr;
				}
				if (x1 < minx[y1]) minx[y1] = x1;
				if (x1 > maxx[y1]) maxx[y1] = x1;

				if (x2 < 0) 
				{
					x2 = 0;
				}
				else if (x2 > clipxr) 
				{
					x2 = clipxr;
				}
				if (x2 < minx[y1]) minx[y1] = x2;
				if (x2 > maxx[y1]) maxx[y1] = x2;
				continue;
			}

			// Scanline conversion of the edge (Bresenham)
			{
				long m = (((long)(x2 - x1)) << 7) / (y2-y1); 
				long cx = 0;

				Int16 *__restrict minxptr = &minx[y1];
				Int16 *__restrict maxxptr = &maxx[y1];

				int rowcount = y2-y1;				
				do 
				{
					int x = x1 + (cx >> 7);
					if (x < 0) 
					{
						x = 0;
					}
					else if (x > clipxr) 
					{
						x = clipxr;
					}
					if (x < *minxptr) *minxptr = x;
					if (x > *maxxptr) *maxxptr = x;
					
					cx += m;
					minxptr++;
					maxxptr++;
				} while (--rowcount >= 0);
			}
		}
	} while(--vertexIdx);

	if (flago == vertexCount || flagu == vertexCount) 
	{
		// Polygon is completely outside of screen
		return 0L;
	}

	const UInt32 colorUInt32 = (((UInt32)colorIdx) << 24) + (((UInt32)colorIdx) << 16) + (((UInt32)colorIdx) << 8) + ((UInt32)colorIdx);

	UInt8 *rowBits = (UInt8 *)drawBits;
	rowBits += ((long)miny * rowBytes);

	Int16 *__restrict minxptr = &minx[miny];
	Int16 *__restrict maxxptr = &maxx[miny];

	for (Int16 y = maxy - miny; y >= 0; y--)
	{
		const Int16 cminx = *minxptr++;
		
		if (cminx == clipxr)
		{
			rowBits += rowBytes;
			continue;
		}
		
		const Int16 cmaxx = *maxxptr++;

		Int16 width = (cmaxx - cminx) + 1;
		
		UInt8 *drawBits = rowBits + cminx;
		
		// Draw short fragments in an ordinary loop
		if (width <= 10)
		{
			while(--width >= 0)
			{
				*drawBits++ = colorIdx;
			}
		}
		else
		{
			// Write the line 4 pixels at a time. Make sure only even memory addresses will be touched.

			switch(cminx & 3)
			{
				case 1:
				*drawBits++ = colorIdx;
				case 2:
				*drawBits++ = colorIdx;
				case 3:
				*drawBits++ = colorIdx;

				width -= 4 - (cminx & 3);
			}
			
			width >>= 2;
							
			while(--width >= 0)
			{
				(*((UInt32 *)drawBits)) = colorUInt32;					
				drawBits += 4;
			}
							
			switch(cmaxx & 3)
			{
				case 2:
				*drawBits++ = colorIdx;
				case 1:
				*drawBits++ = colorIdx;
				case 0:
				*drawBits++ = colorIdx;
			}
		}

		rowBits += rowBytes;
	}

	return (UInt32)0L;
}
