/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SonyClieCanvas.cpp,v 1.5 2003/05/30 12:30:21 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include "Customization.h"

#ifdef SUPPORT_CLIE
#include <PalmOS.h>
#include <BmpGlue.h>
#include "gfx/PalmOSCanvas.h"
// Duplicate definition: One in OS5 SDK, one in Sony SDK
#undef vchrSonyMin
#undef vchrSonyMax
#include <SonyCLIE.h>
#include "gfx/SonyClieCanvas.h"
#include "gfx/SonyClieBlitter.h"
#include "gfx/BlitterSupport.h"
#ifndef MODE_PURITY
#include "gfx/Direct8bppBlitter.h"
#endif
#include "Device.h"
#include "gfx/Display.h"

using namespace prvrazor;


SonyClieCanvas::SonyClieCanvas(UInt16 hrRefnum, Coord width, Coord height, CoordSystem coordSystem) 
	: Canvas(width, height, coordSystem)
{
	this->hrRefnum = hrRefnum;

	// Create double buffer
	UInt16 error;
	if (Display::density == Display::DENSITY_SINGLE)
	{
		offscreenWindow = WinCreateOffscreenWindow(cols, rows, screenFormat, &error);
	}
	else
	{
		offscreenWindow = HRWinCreateOffscreenWindow(hrRefnum, cols, rows, screenFormat, &error);
	}

	if (error)
	{
		Device::panic("Cannot create double buffer.");
	}

	drawWindow = offscreenWindow;
	_drawBitmap = BlitterSupport::getWindowBitmap(drawWindow);
	drawBits = BlitterSupport::getWindowBits(drawWindow);
	BmpGlueGetDimensions(_drawBitmap, NULL, NULL, &rowBytes);

#ifndef MODE_PURITY
	if ((Display::density == Display::DENSITY_SINGLE) && (Display::depth == 8))
	{
		blitter = new Direct8bppBlitter(this);
	}
	else
#endif
	{
		blitter = new SonyClieBlitter(this);
	}
}
	

SonyClieCanvas::SonyClieCanvas(UInt16 hrRefnum) 
	: Canvas(Display::cols, Display::rows, Canvas::COORDS_NATIVE)
{
	this->hrRefnum = hrRefnum;

	offscreenWindow = NULL;
	drawWindow = WinGetDisplayWindow();
	_drawBitmap = BlitterSupport::getWindowBitmap(drawWindow);
	drawBits = BlitterSupport::getWindowBits(drawWindow);
	BmpGlueGetDimensions(_drawBitmap, NULL, NULL, &rowBytes);

#ifndef MODE_PURITY
	if ((Display::density == Display::DENSITY_SINGLE) && (Display::depth == 8))
	{
		blitter = new Direct8bppBlitter(this);
	}
	else
#endif
	{
		blitter = new SonyClieBlitter(this);
	}
}

	
SonyClieCanvas::~SonyClieCanvas()
{
	if (blitter != NULL)
	{
		delete blitter;
	}
	
	if (offscreenWindow != NULL)
	{
		WinDeleteWindow(offscreenWindow, false);
	}
}


void SonyClieCanvas::setOSCanvas() const
{
    WinSetDrawWindow(drawWindow);
}

Boolean SonyClieCanvas::setForegroundColor(const Color color) const
{
    return PalmOSCanvas::_setForegroundColor(color);
}

Boolean SonyClieCanvas::setBackgroundColor(const Color color) const
{
    return PalmOSCanvas::_setBackgroundColor(color);
}


#endif // SUPPORT_CLIE
