/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SonyClieDisplay.cpp,v 1.4 2003/05/30 12:30:21 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#ifdef SUPPORT_CLIE
// Duplicate definition: One in OS5 SDK, one in Sony SDK
#undef vchrSonyMin
#undef vchrSonyMax
#include <SonyCLIE.h>
#include "gfx/SonyClieDisplay.h"
#include "gfx/SonyClieCanvas.h"
#include "Device.h"

using namespace prvrazor;


Boolean SonyClieDisplay::isSupported()
{
	// Sony OS5 devices (e.g. NX70V) could use both APIs, but OS5 is preferred.
	if (Device::supports50)
	{
		return false;
	}
	else
	{
		return SonyClieDisplay::openSonyHRLib();
	}
}


SonyClieDisplay::SonyClieDisplay() : Display()
{
    initPhysicalDisplay();
	displayCanvas = new SonyClieCanvas(sonyHRLibRefNum);
}
	
	
SonyClieDisplay::~SonyClieDisplay()
{
}

	
void SonyClieDisplay::endDrawViewport(RectangleType *bounds)
{
	Display::endDrawViewport(bounds);
}


void SonyClieDisplay::showViewport()
{
	// Copy the screenBuffer
    const Int8 shiftFactor = displayCanvas->shiftFactor;
	if (shiftFactor != 0)
	{
		RectangleType adjustedBounds;	
		adjustedBounds.topLeft.x = copyBounds.topLeft.x << shiftFactor;
		adjustedBounds.topLeft.y = copyBounds.topLeft.y << shiftFactor;
		adjustedBounds.extent.x = copyBounds.extent.x << shiftFactor;
		adjustedBounds.extent.y = copyBounds.extent.y << shiftFactor;

		HRWinCopyRectangle (SonyClieDisplay::sonyHRLibRefNum, 
			viewportCanvas->drawWindow, displayCanvas->drawWindow, 
			&adjustedBounds, 
			adjustedBounds.topLeft.x, 
			adjustedBounds.topLeft.y + (viewport.region.topLeft.y << 1), 
			winPaint);
	}
	else
	{
		HRWinCopyRectangle (SonyClieDisplay::sonyHRLibRefNum, 
			viewportCanvas->drawWindow, displayCanvas->drawWindow, 
			&copyBounds, 
			copyBounds.topLeft.x, 
			copyBounds.topLeft.y + viewport.region.topLeft.y, 
			winPaint);
	}
}


void SonyClieDisplay::initDisplay(const DisplayProperties& displayProperties)
{
    displayCanvas->setCoordSystem(displayProperties.coordSystem);
}


void SonyClieDisplay::initPhysicalDisplay()
{
    MemHandle resH = DmGetResource( (DmResType)'Tbmt', 1000);
    ErrNonFatalDisplayIf(!resH, "Bitmap metadata not found!");
    void *resP = MemHandleLock(resH);

    UInt8 *metaBytes = static_cast<UInt8 *>(resP);

    // Determine supported depths
	UInt32 supportedDepths;
	WinScreenMode(winScreenModeGetSupportedDepths, NULL, NULL, &supportedDepths, NULL);		

    ///////////////////////////////////////////////////////////////////////////////////
    // Iterate over the resource metadata sets, trying to find a match
    ///////////////////////////////////////////////////////////////////////////////////
    bool found = false;    
    while ((*metaBytes != 0x00) && !found)
    {
        const UInt8 desiredDensity = *metaBytes;
        const UInt8 desiredDepth = *(metaBytes + 1);

        if (supportedDepths & (1 << (desiredDepth - 1)))
        {
            // Set selected density
            switch(desiredDensity)
            {
                case 1:
                Display::cols = Display::rows = 160;
                break;
                case 2:
                Display::cols = Display::rows = 320;
                break;
                default:
                ErrNonFatalDisplay("Invalid density in bitmap metadata");
            }

            UInt32 cols = Display::cols;
            UInt32 rows = Display::rows;
    		const Err error = HRWinScreenMode(SonyClieDisplay::sonyHRLibRefNum, winScreenModeSet, &cols, &rows, NULL, NULL);
            Display::density = static_cast<Display::Density>(desiredDensity);

        	// Set selected display depth	
            Display::depth = desiredDepth;

        	UInt32 currentDisplayDepth;
        	WinScreenMode(winScreenModeGet, NULL, NULL, &currentDisplayDepth, NULL);
        	if (currentDisplayDepth != Display::depth)
        	{
        		const Err error = WinScreenMode(winScreenModeSet, NULL, NULL, &Display::depth, NULL);
                if (error != errNone)
                {
             		Device::panic("Unable to set display depth");
             	}
         	}


        	// Is direct color supported?
        	// See http://oasis.palm.com/dev/kb/faq/2449.cfm
        	Display::directColor = false;
        	if ((Display::depth >= 16) && Device::supports35 && Device::supportsTrap(sysTrapWinSetForeColorRGB))
        	{
        		Display::directColor = true;
        	}

            Display::colorMode = (Display::depth >= 8);

            found = true;
        }                

        metaBytes += 2;
    }
    
    
    MemPtrUnlock(resP);
    DmReleaseResource(resH);
    
    if (!found)
    {
        Device::panic("This version does not support your device's display properties.");
    }
}





void SonyClieDisplay::resetDisplay()
{
	HRWinScreenMode(sonyHRLibRefNum, winScreenModeSetToDefaults, NULL, NULL, NULL, NULL);
	SonyClieDisplay::closeSonyHRLib();	
}



void SonyClieDisplay::createViewportCanvas(const ViewportProperties& viewport)
{
	viewportCanvas = 
	    new SonyClieCanvas(sonyHRLibRefNum, viewport.region.extent.x, viewport.region.extent.y, displayCanvas->coordSystem);
}


void SonyClieDisplay::captureDisplay()
{
}


void SonyClieDisplay::releaseDisplay()
{
}



Boolean SonyClieDisplay::openSonyHRLib()
{
	// Open Sony Hires-Library. 
	Err error;
	SonySysFtrSysInfoP sonySysFtrSysInfoP;
	if ((error = FtrGet(sonySysFtrCreator, sonySysFtrNumSysInfoP, reinterpret_cast<UInt32*>(&sonySysFtrSysInfoP)))) 
	{			
		/* Not CLIE: maybe not available */
		return false;
	} 
	else 
	{
		if (sonySysFtrSysInfoP->libr & sonySysFtrSysInfoLibrHR) 
		{
			/* HR available */
			if ((error = SysLibFind(sonySysLibNameHR, &SonyClieDisplay::sonyHRLibRefNum)))
			{
				if (error == sysErrLibNotFound) 
				{
					/* couldn't find lib */
					error = SysLibLoad( 'libr', sonySysFileCHRLib, &SonyClieDisplay::sonyHRLibRefNum);
				}
			}

			if (error) 
			{
				return false;
			}
			else
			{
				/* Now we can use HR lib */
				error = HROpen(SonyClieDisplay::sonyHRLibRefNum);
				if (error == errNone)
				{
					return true;
				}
			}
		}
	}	

	return false;
}


void SonyClieDisplay::closeSonyHRLib()
{
	// Sony HRClose does all the work. No SysLibRemove allowed afterwards!
	HRClose(SonyClieDisplay::sonyHRLibRefNum);
}


UInt16 SonyClieDisplay::sonyHRLibRefNum;

#endif // SUPPORT_CLIE
