/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: DigitalController.h,v 1.3 2003/03/17 22:18:20 teacy Exp $
 *
 * Copyright (c) 2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef DIGITAL_CONTROLLER_H
#define DIGITAL_CONTROLLER_H

#include "RazorSections.h"

#include "input/Controller.h"


/**
 * A digital controller can steer in 8 directions, 
 * and has a couple of buttons that can be pushed.
 */
class DigitalController : public Controller
{
	public:
	/**
	 * Return the current direction of the controller
	 *
	 * @return an arbitrary bitwise OR-ed combination of the DIR_xxx constants
	 */
	virtual int getDirection() const SEC_RAZOR = 0;

	/**
	 * Return the number of supported buttons
	 */
	virtual int getButtonCount() const SEC_RAZOR = 0;

	/**
	 * Return whether a button is currently pressed
	 */
	virtual bool isButtonPressed(int buttonIndex) const SEC_RAZOR = 0;

	static const int DIR_RELEASED = 0;
	static const int DIR_UP       = 1;
	static const int DIR_DOWN     = 2;
	static const int DIR_RIGHT    = 4;
	static const int DIR_LEFT     = 8;

	virtual ~DigitalController() = 0;

	protected:
	DigitalController() SEC_RAZOR;
};


#endif
