/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: BlitterSupport.h,v 1.1 2003/01/26 10:45:56 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef BLITTER_SUPPORT_H
#define BLITTER_SUPPORT_H

#include <PalmOS.h>
#include "RazorSections.h"
#include "Customization.h"

namespace prvrazor
{

typedef struct RscBitmapTypeV1
 	{
 	Int16  				width;
	Int16  				height;
	UInt16  			rowBytes;
	BitmapFlagsType	flags;					
	UInt8				pixelSize;
	UInt8				version;					
	
	UInt16	 			nextDepthOffset;	
	UInt16				reserved[2];	
	}
RscBitmapTypeV1;


typedef struct RscBitmapTypeV2
{
 	Int16  				width;
	Int16  				height;
	UInt16  			rowBytes;
	BitmapFlagsType	flags;
	UInt8				pixelSize;
	UInt8				version;					
	
	UInt16	 			nextDepthOffset;	
	UInt8				transparentIndex;	
	UInt8				compressionType;
	UInt16	 			reserved;
}
RscBitmapTypeV2;

typedef struct RscBitmapTypeV3
{
	Int16  				width;
	Int16  				height;
	UInt16  			rowBytes;
	BitmapFlagsType	flags;		
	UInt8				pixelSize;			
	UInt8				version;			
	
	UInt8				size;
	UInt8				pixelFormat;	
	UInt8				unused;
	UInt8				compressionType;
	UInt16				density;			
	UInt32				transparentValue;	
	UInt32				nextBitmapOffset;
}
RscBitmapTypeV3;


/**
 * Various support functions which are req'd by several Blitter implementations.
 */
class BlitterSupport
{
	public:
	/**
	 * Make a copy of a window's bitmap in feature memory.
	 */
	static Err copyBitmap(WinHandle sourceWindow, UInt16 ftrId, Coord width, Coord height, BitmapType **bitmapType, void **bitmapBits) SEC_RAZOR_INIT;

	/**
	 * Create a new BitmapTypeV2 structure that points to the bits of an existing BitmapTypeV3 bitmap
	 */
	static BitmapType *createIndirectV2Copy(const BitmapType *bitmapP, UInt32 desiredDepth) SEC_RAZOR_INIT;

	/**
	 * Find an appropriate high density member of a bitmap family.
	 */
	static const BitmapType *getHighDensityMember(const BitmapType *bitmapP, UInt32 desiredDepth) SEC_RAZOR_INIT;

	/**
	 * Get the pixelsize in bits of the supplied bitmap.
	 */
	static Int16 getPixelSize(const BitmapType *bitmapP);	


	/**
	 * Is the bitmap compressed?
	 */
	static Boolean isCompressed(const BitmapType *bitmapP);


	/**
	 * Get the address of the bitmap which underlies a window.
	 */
	static void *getWindowBits(WinHandle window);


	/**
	 * Get the BitmapType structure which underlies a window.
	 */
	static BitmapType *getWindowBitmap(WinHandle window);

	private:
	static RscBitmapTypeV2 *createBitmapTypeV2
	(
		Int16 width, Int16 height,
		Int16 pixelSize, 
		const UInt8 *bits
	) SEC_RAZOR_INIT;
};
}

#endif
