/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SonyClieBlitter.h,v 1.4 2003/05/29 15:46:33 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef SONYCLIE_BLITTER_H
#define SONYCLIE_BLITTER_H

#include "Customization.h"
#ifdef SUPPORT_CLIE

#include "RazorSections.h"
#include "gfx/Blitter.h"
#include "gfx/Canvas.h"


namespace prvrazor
{


class SonyClieCanvas;


class SonyClieTransparentBitmap
{
	WinHandle bitmap;
	WinHandle mask;

	friend class SonyClieTransparentBitmapArray;
};


class SonyClieTransparentBitmapArray : public BitmapArray
{
	public:
	SonyClieTransparentBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID, UInt16 maskID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~SonyClieTransparentBitmapArray() SEC_RAZOR_INIT;

	private:
	const SonyClieTransparentBitmap *const bitmaps;	
};



class SonyClieSolidBitmap
{
	BitmapType *bitmap;
	BitmapType *v3Bitmap;  // Not really a V3 bitmap, but a V2 copy of a V3 bitmap
	MemHandle bitmapResH;

	friend class SonyClieSolidBitmapArray;
};

class SonyClieSolidBitmapArray : public BitmapArray
{
	public:
	SonyClieSolidBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~SonyClieSolidBitmapArray() SEC_RAZOR_INIT;

	private:
	const SonyClieSolidBitmap *const bitmaps;	
};


class SonyClieSaveBuffer
{
	WinHandle buffer;
	Coord x;
	Coord y;
	
	friend class SonyClieSaveBufferArray;
};


class SonyClieSaveBufferArray : public SaveBufferArray
{
	public:
    SonyClieSaveBufferArray(const Canvas *canvas, UInt16 numBuffers, Coord width, Coord height) SEC_RAZOR_INIT;
    virtual ~SonyClieSaveBufferArray() SEC_RAZOR_INIT;
    virtual void save(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;
    virtual void restore(const Canvas *canvas, UInt16 index) const;

    private:
    SonyClieSaveBuffer *const buffers;
};


/**
 * Implementation of the Blitter functionality for Sony Clie.
 */
class SonyClieBlitter : public Blitter
{
	public:
	SonyClieBlitter(const SonyClieCanvas *canvas) SEC_RAZOR_INIT;
	~SonyClieBlitter() SEC_RAZOR_INIT;

	const BitmapArray *prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const SEC_RAZOR_INIT;

	SaveBufferArray *prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const SEC_RAZOR_INIT;
	
	void drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const;	

	void fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const;

	private:
	UInt16 hrRefnum;
};

}

#endif  // SUPPORT_CLIE
#endif
